@@ -9,7 +9,7 @@ from tmcp import TMCP_VERSION
 from .message import TMCPMessage
 
 
-MAX_PACKETS_PER_TICK: int = 50
+MAX_PACKETS_PER_TICK: int = 30
 TIME_BETWEEN_MESSAGES: float = 0.1
 
 
@@ -49,20 +49,37 @@ class TMCPHandler:
         self.matchcomms: MatchcommsClient = agent.matchcomms
         self.index: int = agent.index
         self.team: int = agent.team
-        self.last_time: float = 0.0
+        self.last_time: float = 0.0 
+        self.enabled: bool = True
+
+    def disable(self):
+        """Disable the handler. It will not send or receive any messages."""
+        self.enabled = False
 
     def send(self, message: TMCPMessage) -> bool:
         """Send a TMCPMessage over match comms. Will not send messages if they are coming too quickly.
         Returns whether a message was sent."""
+
+        # If disabled, pretend all messages are sent.
+        # This is done so that people don't try to resend messages if the handler is disabled.
+        if not self.enabled:
+            return True
+
         current_time = perf_counter()
         if current_time - self.last_time < TIME_BETWEEN_MESSAGES:
             return False
+
         self.matchcomms.outgoing_broadcast.put_nowait(message.to_dict())
         self.last_time: float = current_time
         return True
 
     def recv(self) -> List[TMCPMessage]:
         messages = []
+
+        # Return empty message list if disabled.
+        if not self.enabled:
+            return messages
+
         # Receive messages until we reach the maximum packets per tick or the queue is empty.
         for _ in range(MAX_PACKETS_PER_TICK):
             try:
@@ -113,8 +130,20 @@ class TMCPHandler:
             msg = TMCPMessage.demo_action(self.team, self.index, target, time)
         return self.send(msg)
 
-    def send_wait_action(self) -> bool:
-        """The bot is waiting for a chance to go for the ball or boost.
+    def send_wait_action(self, ready: float) -> bool:
+        """The bot is waiting for a chance to go for the ball.
         Some examples are positioning (retreating/shadowing) and recovering.
+
+        `ready` - A value of -1 signifies that the bot is NOT ready to go for the ball.
+        Otherwise, the ready value indicates the projected intercept time in game time
+        if the bot was to go for the ball.
         """
-        return self.send(TMCPMessage.wait_action(self.team, self.index))
+        return self.send(TMCPMessage.wait_action(self.team, self.index, ready))
+
+    def send_defend_action(self) -> bool:
+        """The bot is in a position to defend the goal and is not planning to move up.
+        If the bot decides to leave net, signal this using either "BALL" (if going for a touch) or "WAIT" (if moving upfield).
+
+        A bot should use "DEFEND" to let its teammates know it is safe to move up a bit without worrying about an open net.
+        """
+        return self.send(TMCPMessage.defend_action(self.team, self.index))
